//INCLUDES
#include <p18f4550.h>
#include "system\typedefs.h"
#include "system\usb\usb.h"

//DEFINES
#define TAILLE_BUFFER 128
//Max 128 ???
#define LED0 PORTBbits.RB0	//Voie 1
#define LED1 PORTBbits.RB3	//Voie 2
#define LED2 PORTCbits.RC6	//Connexion

//VARIABLES
char input_buffer[2];
char output_buffer[TAILLE_BUFFER];
char useAN0, useAN1, etat, tour;
//char buffer;
//FONCTIONS
static void InitializeSystem(void);
void USBTasks(void);
void InitializeUSART(void);
void CAN(void);

//MAIN
void main(void){
    InitializeSystem();
    while(1){
        USBTasks();         // USB Tasks
        CAN();        // See user\user.c & .h
    }
}

// CONVERSION ANALOGIQUE NUMERIQUE
void CAN(void){
	char i;
	if((usb_device_state < CONFIGURED_STATE)||(UCONbits.SUSPND==1)) return;
	
	if(etat == 0) {	//Etat Normal.
		if(getsUSBUSART(input_buffer,2)){	//Recoie les donnes envoyes par le PC
			switch(input_buffer[0]) {
				case '0' :		//Ne rien faire
					useAN0 = 0;
					useAN1 = 0;
					break;
				case '1' :		//Acquisition voie 1
					useAN0 = 1;
					useAN1 = 0;
					break;
				case '2' :		//Acquisition voie 2
					useAN0 = 0;
					useAN1 = 1;
					break;
				case '3' :		//Acquisition voie 1 et 2
					useAN0 = 1;
					useAN1 = 1;
					break;
				case 'c' :		//Allume la LED CONNEXION
					LED2 = 1;
					useAN0 = 0;
					useAN1 = 0;
					break;
				case 'd' :		//Eteint la LED CONNEXION
					LED2 = 0;
					useAN0 = 0;
					useAN1 = 0;
					break;
				default :
					break;
			}
			LED0 = useAN0;
			LED1 = useAN1;
			switch (input_buffer[1]){
				case '1' :
					ADCON2 = 0b10110000;
					break;
				case '2' :
					ADCON2 = 0b10110100;
					break;
				case '3' :
					ADCON2 = 0b10110001;
					break;
				case '4' :
					ADCON2 = 0b10110101;
					break;
				case '5' :
					ADCON2 = 0b10110010;
					break;
				case '6' :
					ADCON2 = 0b10110110;
					break;
				default :
					break;
			}
		}
		
		if (useAN0 || useAN1) {	//Si l'acquisition est lance sur une des 2 voies
			for(i=0; i<(TAILLE_BUFFER-2); i+=2) {		

				if (useAN0 && !tour) {	//Acquisition sur la voie 1
					ADCON0 = 0x01;	//Turn on AD module, select AN0
					ADCON0 = 0x03;  //Lancement de l'aquisition.
				} else if (useAN1 && tour){	//Acquisition sur la voie 2
					ADCON0 = 0x05;	//Turn on AD module, select AN1
					ADCON0 = 0x07;  //Lancement de l'aquisition.
				}
		
				while (ADCON0bits.GO_DONE);	//Attend la fin de la conversion
				
				
				output_buffer[i] = ADRESH;	//ADRES : Rsultat de la conversion
				output_buffer[i+1] = ADRESL;
				if (tour) {
					output_buffer[i] +=64;	//Voie 1
				}	
				else if (!tour) {
					output_buffer[i] +=128;	//Voie 0
				}

				if (useAN0 && useAN1) {	//Permet d'alterner les acquisitions sur les 2 voies
					tour = !tour;		//si elle sont actives toutes les deux.
				} else if (useAN0 || useAN1) {
					tour = useAN1;
				}
			}
			etat = 1;	//Il y a des donnes  envoyer.
			useAN0 = 0;
			useAN1 = 0;
		}
	}
	
	if (etat == 1) {	// Si il y a des donnes  envoyer
		if(mUSBUSARTIsTxTrfReady()){	// et si le derner envoi est termin
			//ECHANGE DES BUFFERS
			
			mUSBUSARTTxRam((byte*)output_buffer, TAILLE_BUFFER);	//Envoi des donns
			CDCTxService();
			etat = 0;
		} 			
	}
}

static void InitializeSystem(void){
	TRISBbits.TRISB3 = 0;	// Configure ces 3 pin en sortie.
	TRISBbits.TRISB0 = 0;
	TRISCbits.TRISC6 = 0;
    mInitializeUSBDriver();         // voir usbdrv.h

	//Config entre analogique.
	TRISA = 0b00001111;	//PORTA en tant que sortie, AN0 et AN1 en entre
	//Utilisation de Vref+ et Vref-.
	ADCON1 = 0b00111101;	//PORTA en mode numrique, sauf AN0 et AN1 en analog/
	//Utilise Vref+ et Vref-
	ADCON2 = 0b10001101;	// 2TAD, Fosc/16
	
	//Initialisation des variables.
	etat = 0;
	useAN0 = 0;
	useAN1 = 0;
	tour = 0;
//	buffer = 0;
	LED0 = 0;
	LED1 = 0;
	LED2 = 0;
}

// TACHES USB
void USBTasks(void){
    USBCheckBusStatus();           
    if(UCFGbits.UTEYE!=1)
        USBDriverService();
    
    #if defined(USB_USE_CDC)
    CDCTxService();
    #endif
}